(******************************************************************************
 *                             Core-extended                                  *
 *                                                                            *
 * Copyright (C) 2008- Jane Street Holding, LLC                               *
 *    Contact: opensource@janestreet.com                                      *
 *    WWW: http://www.janestreet.com/ocaml                                    *
 *                                                                            *
 *                                                                            *
 * This library is free software; you can redistribute it and/or              *
 * modify it under the terms of the GNU Lesser General Public                 *
 * License as published by the Free Software Foundation; either               *
 * version 2 of the License, or (at your option) any later version.           *
 *                                                                            *
 * This library is distributed in the hope that it will be useful,            *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of             *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU          *
 * Lesser General Public License for more details.                            *
 *                                                                            *
 * You should have received a copy of the GNU Lesser General Public           *
 * License along with this library; if not, write to the Free Software        *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  *
 *                                                                            *
 ******************************************************************************)

type t = 
  | Realtime
  | Monotonic
  | Process_cpu
  | Process_thread

let all = [
  Realtime;
  Monotonic;
  Process_cpu;
  Process_thread;
]

let to_string t =
  match t with
  | Realtime       -> "Realtime"
  | Monotonic      -> "Monotonic"
  | Process_cpu    -> "Process_cpu"
  | Process_thread -> "Process_thread"

external getres : t -> int = "caml_clock_getres"
external gettime : t -> int = "caml_clock_gettime"
(*external nanosleep : t -> int -> unit = "caml_clock_nanosleep"*)

let min_interval t =
  let canary_val = 1_000_000 in
  let current_min = ref canary_val in
  for i = 1 to 10_000 do
    let t1 = gettime t in
    let t2 = gettime t in
    if t1 <> t2 && t2 > t1 then current_min := min (t2 - t1) !current_min
  done;

  if !current_min <> canary_val then !current_min
  else failwith (Printf.sprintf "unable to calculate min_interval for %s" (to_string t))

let mean_gettime_cost ~measure ~using =
  assert (getres Process_cpu = 1);
  let count = 10_000_000 in
  let start = gettime using in
  for i = 1 to count do
    ignore (gettime measure);
  done;
  let stop = gettime using in
  (stop - start) / count
