(******************************************************************************
 *                             Core-extended                                  *
 *                                                                            *
 * Copyright (C) 2008- Jane Street Holding, LLC                               *
 *    Contact: opensource@janestreet.com                                      *
 *    WWW: http://www.janestreet.com/ocaml                                    *
 *                                                                            *
 *                                                                            *
 * This library is free software; you can redistribute it and/or              *
 * modify it under the terms of the GNU Lesser General Public                 *
 * License as published by the Free Software Foundation; either               *
 * version 2 of the License, or (at your option) any later version.           *
 *                                                                            *
 * This library is distributed in the hope that it will be useful,            *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of             *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU          *
 * Lesser General Public License for more details.                            *
 *                                                                            *
 * You should have received a copy of the GNU Lesser General Public           *
 * License along with this library; if not, write to the Free Software        *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  *
 *                                                                            *
 ******************************************************************************)

open Core.Std ;;

(** [with_conn] opens a connection to the TCP service on [host] [port] and
  if successful calls [f] with In_channel.t and Out_channel.t as arguments.
  The return value of [f] will be returned by [with_conn].

  [with_conn] raises Connect_timeout if the connection attempt times out.
  Unix_error will be raised if there are any other connect errors.
  Any exceptions raised by [f] will be re-raised, which may include
  Unix_errors raised by underlying channel read/write calls.

  The [In_channel.t], [Out_channel.t], and underlying socket
  will be closed by with_conn before it returns.
 *)

exception Connect_timeout

val with_connection :
    ?timeout:Time.Span.t
  -> host:string
  -> port:int
  -> f:(In_channel.t -> Out_channel.t -> 'a)
  -> unit
  -> 'a

